#ifndef __IUF_GAMESESSION
#define __IUF_GAMESESSION

#include <vcg/Point3.h>
#include <list>
#include "GameSection.h"
#include "GameObj.h"
#include "GameObjTimed.h"
#include "Bullet.h"
#include "Ast.h"
#include "Ship.h"
#include "Stars.h"


/** This class represent a Game session.
It contain all the data needed during the game play, 
active game objects, score, lives and so on.
Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.1s Modified Shoot to manage multiple shots .
\item 0.3 Added collision detection between ship and asteroids,
 \Ref(AstShipCollision), and between bullets and Asteroids,
 \Ref{BulletAstCollision}. Modified \Ref{Update} to manage the 
 effect of the various collisions (death of the player, explosion 
 and splitting of the asteroid).
\item 0.3s Added PauseFlag and implemented Pause()
\item 0.4 Subclassed from GameSection, bring in all the callbacks and the opengl intialization.
Now at startup there is a fixed number of Large Asteroids, and all 
the smaller pieces are created (timeconsuming operation) at startup 
instead of at running time (in the previous version game stopped when 
there were many explosions)
\item 0.4s added starred background
\item 0.5 Added (at last!) score, lives,  refill of asteroid and 
no more start inside an asteroid. Audio effects are started here in 
function Shoot, AstShipCollision and BulletAstCollision..
\end{itemize}
}


*/
using namespace vcg;
using namespace std;
class GameSession:public GameSection
{
 public:
   /** Default Number of Asteroids at startup
	 @name AstNum=8
	 */
	 enum {AstNum=8};

	 GameSession()
	 {
		 PauseFlag=false;
		 SpaceSize=100;
	 }
 
 /** The list of all the active bullets
 @type list<Bullet *> */
	 list<Bullet *> ActiveBullet;
 
 /** The list of all the active Asteroid 
 @type list<Ast*> */
	 list<Ast *> ActiveAst;
 
 /** The list of all the generic active objects 
 @type list<GameObj *> */
	 list<GameObj *> ActiveObj;
 
	/// The Starred Background
	Stars SB;
	
  /// The only Spaceship in game.
  Ship *S;
 
	/** This function updates All the active objects.
	This is done by traversing the three list (ActiveAst, 
	ActiveBullet and ActiveObj) and calling the Update virtual 
	function for each of them. If an object become inactive 
	after an update it is removed by the list and deallocated here.
	During the traversal of ActiveAst list we also check for 
	\Ref{AstShipCollision}.
	During the traversal of ActiveBullet list we check 
	for \Ref{BulletAstCollision} Note that this function traverse 
	again the ActiveAst list (quadratically!). 
*/
 	void Update(int dt);
 
  /** This function draws all the active objects.
	The ActiveObj list (the one containing explosions) 
	is drawn as the last list to correctly draw the explosion 
	transparency effects.
	*/
  int Draw();
 
  /// This function clip the position of an object against the screen boundary
  void Clip(Point3f &p)
  {
	 if(p.x() > SpaceSize) p.x() -=2*SpaceSize; 
   if(p.y() > SpaceSize) p.y() -=2*SpaceSize; 
   if(p.x() <-SpaceSize) p.x() +=2*SpaceSize; 
   if(p.y() <-SpaceSize) p.y() +=2*SpaceSize; 
 };
 
  /** Shoot a bullet.
 What kind of (and how many!) bullet is shot depends on the current type of 
 ship stored in member S;
 */
 void Shoot();

 /** Manage key pression.
 */
 void Key(unsigned char key);

 /** Manage special key pression.
It should start the thrust (up arrow), 
the rotation (left/right arrow) and side thrust 
(alt+left/right) of  the ship.
 */
void SKey(int key);
void SKeyUp(int key);
void Reshape(int w, int h);

 /** Start the game.
 From version 0.4 it does also all the opengl init stuff, and 
 the creation of all asteroids (larger and smaller)

 */
 void GameSession::Init(int _score=0, int _lives=3);
 /// Pause the game
 void Pause ()
 {PauseFlag =! PauseFlag;}

 /// Game Over;
 void End();

 ///The size of the Game Space (in mt)
 int SpaceSize;

 /** Check the collision between the asteroids and the ship
 If the ship is hit by an asteroid the ship is inactivated 
 (e.g. it will not be drawn) for a couple of seconds by setting 
 the deadpause member variable.
	*/
  void AstShipCollision(Ast *a);

 /** Check the collision between a bullet and all the asteroids.
 If an asteroid is hit the bullet and the asteroid are deactivated,
 a new explosion is added to the ActiveObj list and two new smaller
 asteroids are extracted from the prebuilt list of asteroids.
 */
 void BulletAstCollision(Bullet *b);

	/** Used to delay the creation of the new ship. 
	After a ship-asteroid collision the AstShipCollision
	function set this to wait for two secs.
	*/
	int deadpause;	

	/** Used to delay the end of level.
	Once we have destroyed the last asteroids, update
	function set this to wait for two secs.
	*/
	int endlevelpause;
/// True if the game is paused;
	bool PauseFlag;

/// the Current Score
	int Score;

/// Current number of lives
	int Lives	;

};

#endif // ifndef __IUF_GAMESESSION